@@ -4,17 +4,24 @@ localization data.
 """
 
 import gettext
-import json
 import importlib
+import json
 import os
-import subprocess
+import re
 import sys
 import traceback
+from functools import lru_cache
+from typing import Dict, Pattern, Tuple
 
 import babel
-import pkg_resources
 from packaging.version import parse as parse_version
 
+# See compatibility note on `group` keyword in https://docs.python.org/3/library/importlib.metadata.html#entry-points
+if sys.version_info < (3, 10):
+    from importlib_metadata import entry_points
+else:
+    from importlib.metadata import entry_points
+
 # Entry points
 JUPYTERLAB_LANGUAGEPACK_ENTRY = "jupyterlab.languagepack"
 JUPYTERLAB_LOCALE_ENTRY = "jupyterlab.locale"
@@ -23,6 +30,45 @@ JUPYTERLAB_LOCALE_ENTRY = "jupyterlab.locale"
 DEFAULT_LOCALE = "en"
 LOCALE_DIR = "locale"
 LC_MESSAGES_DIR = "LC_MESSAGES"
+DEFAULT_DOMAIN = "jupyterlab"
+L10N_SCHEMA_NAME = "@jupyterlab/translation-extension:plugin"
+PY37_OR_LOWER = sys.version_info[:2] <= (3, 7)
+
+_default_schema_context = "schema"
+_default_settings_context = "settings"
+_lab_i18n_config = "jupyter.lab.internationalization"
+
+# mapping of schema translatable string selectors to translation context
+DEFAULT_SCHEMA_SELECTORS = {
+    "properties/.*/title": _default_settings_context,
+    "properties/.*/description": _default_settings_context,
+    "definitions/.*/properties/.*/title": _default_settings_context,
+    "definitions/.*/properties/.*/description": _default_settings_context,
+    "title": _default_schema_context,
+    "description": _default_schema_context,
+    # JupyterLab-specific
+    r"jupyter\.lab\.setting-icon-label": _default_settings_context,
+    r"jupyter\.lab\.menus/.*/label": "menu",
+    r"jupyter\.lab\.toolbars/.*/label": "toolbar",
+}
+
+
+@lru_cache()
+def _get_default_schema_selectors() -> Dict[Pattern, str]:
+    return {
+        re.compile("^/" + pattern + "$"): context
+        for pattern, context in DEFAULT_SCHEMA_SELECTORS.items()
+    }
+
+
+def _prepare_schema_patterns(schema: dict) -> Dict[Pattern, str]:
+    return {
+        **_get_default_schema_selectors(),
+        **{
+            re.compile("^/" + selector + "$"): _default_schema_context
+            for selector in schema.get(_lab_i18n_config, {}).get("selectors", [])
+        },
+    }
 
 
 # --- Private process helpers
@@ -45,7 +91,7 @@ def _get_installed_language_pack_locales():
     """
     data = {}
     messages = []
-    for entry_point in pkg_resources.iter_entry_points(JUPYTERLAB_LANGUAGEPACK_ENTRY):
+    for entry_point in entry_points(group=JUPYTERLAB_LANGUAGEPACK_ENTRY):
         try:
             data[entry_point.name] = os.path.dirname(entry_point.load().__file__)
         except Exception:
@@ -74,7 +120,7 @@ def _get_installed_package_locales():
     """
     data = {}
     messages = []
-    for entry_point in pkg_resources.iter_entry_points(JUPYTERLAB_LOCALE_ENTRY):
+    for entry_point in entry_points(group=JUPYTERLAB_LOCALE_ENTRY):
         try:
             data[entry_point.name] = os.path.dirname(entry_point.load().__file__)
         except Exception:
@@ -107,31 +153,6 @@ def _main():
 
 # --- Helpers
 # ----------------------------------------------------------------------------
-def run_process_and_parse(cmd: list):
-    """
-    Run a list of commands and return the result parsed form stdout.
-
-    Parameters
-    ----------
-    cmd: list
-        List of commands
-
-    Returns
-    -------
-    tuple
-        A tuple in the form `(result_dict, message)`.
-    """
-    result = {"data": {}, "message": ""}
-    try:
-        p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        stdout, stderr = p.communicate()
-        result = json.loads(stdout.decode('utf-8'))
-    except Exception:
-        result["message"] = traceback.format_exc() + "\n" + repr(stderr.decode('utf-8'))
-
-    return result["data"], result["message"]
-
-
 def is_valid_locale(locale: str) -> bool:
     """
     Check if a `locale` value is valid.
@@ -161,14 +182,16 @@ def is_valid_locale(locale: str) -> bool:
         babel.Locale.parse(locale)
         valid = True
     except babel.core.UnknownLocaleError:
+        # no-op
         pass
     except ValueError:
+        # no-op
         pass
 
     return valid
 
 
-def get_display_name(locale: str, display_locale: str=DEFAULT_LOCALE) -> str:
+def get_display_name(locale: str, display_locale: str = DEFAULT_LOCALE) -> str:
     """
     Return the language name to use with a `display_locale` for a given language locale.
 
@@ -187,7 +210,10 @@ def get_display_name(locale: str, display_locale: str=DEFAULT_LOCALE) -> str:
     locale = locale if is_valid_locale(locale) else DEFAULT_LOCALE
     display_locale = display_locale if is_valid_locale(display_locale) else DEFAULT_LOCALE
     loc = babel.Locale.parse(locale)
-    return loc.get_display_name(display_locale).capitalize()
+    display_name = loc.get_display_name(display_locale)
+    if display_name:
+        display_name = display_name[0].upper() + display_name[1:]
+    return display_name
 
 
 def merge_locale_data(language_pack_locale_data, package_locale_data):
@@ -227,23 +253,24 @@ def merge_locale_data(language_pack_locale_data, package_locale_data):
     return result
 
 
-def get_installed_packages_locale(locale: str) -> dict:
+def get_installed_packages_locale(locale: str) -> Tuple[dict, str]:
     """
     Get all jupyterlab extensions installed that contain locale data.
 
     Returns
     -------
-    dict
-        Ordered list of available language packs.
-        >>> {"package-name": locale_data, ...}
+    tuple
+        A tuple in the form `(locale_data_dict, message)`,
+        where the `locale_data_dict` is an ordered list
+        of available language packs:
+            >>> {"package-name": locale_data, ...}
 
     Examples
     --------
     - `entry_points={"jupyterlab.locale": "package-name = package_module"}`
     - `entry_points={"jupyterlab.locale": "jupyterlab-git = jupyterlab_git"}`
     """
-    cmd = [sys.executable, __file__, "_get_installed_package_locales"]
-    found_package_locales, message = run_process_and_parse(cmd)
+    found_package_locales, message = _get_installed_package_locales()
     packages_locale_data = {}
     messages = message.split("\n")
     if not message:
@@ -265,11 +292,11 @@ def get_installed_packages_locale(locale: str) -> dict:
                     locale_path,
                     locales[locale.lower()],
                     LC_MESSAGES_DIR,
-                    "{0}.json".format(package_name),
+                    f"{package_name}.json",
                 )
                 if os.path.isfile(locale_json_path):
                     try:
-                        with open(locale_json_path, "r", encoding='utf-8') as fh:
+                        with open(locale_json_path, encoding="utf-8") as fh:
                             packages_locale_data[package_name] = json.load(fh)
                     except Exception:
                         messages.append(traceback.format_exc())
@@ -279,7 +306,7 @@ def get_installed_packages_locale(locale: str) -> dict:
 
 # --- API
 # ----------------------------------------------------------------------------
-def get_language_packs(display_locale: str=DEFAULT_LOCALE) -> tuple:
+def get_language_packs(display_locale: str = DEFAULT_LOCALE) -> Tuple[dict, str]:
     """
     Return the available language packs installed in the system.
 
@@ -295,15 +322,8 @@ def get_language_packs(display_locale: str=DEFAULT_LOCALE) -> tuple:
     -------
     tuple
         A tuple in the form `(locale_data_dict, message)`.
-
-    Notes
-    -----
-    We call `_get_installed_language_pack_locales` via a subprocess to
-    guarantee the results represent the most up-to-date entry point
-    information, which seems to be defined on interpreter startup.
     """
-    cmd = [sys.executable, __file__, "_get_installed_language_pack_locales"]
-    found_locales, message = run_process_and_parse(cmd)
+    found_locales, message = _get_installed_language_pack_locales()
     locales = {}
     messages = message.split("\n")
     if not message:
@@ -330,7 +350,7 @@ def get_language_packs(display_locale: str=DEFAULT_LOCALE) -> tuple:
             }
 
         if invalid_locales:
-            messages.append("The following locales are invalid: {0}!".format(invalid_locales))
+            messages.append(f"The following locales are invalid: {invalid_locales}!")
 
     return locales, "\n".join(messages)
 
@@ -351,8 +371,7 @@ def get_language_pack(locale: str) -> tuple:
     guarantee the results represent the most up-to-date entry point
     information, which seems to be defined on interpreter startup.
     """
-    cmd = [sys.executable, __file__, "_get_installed_language_pack_locales"]
-    found_locales, message = run_process_and_parse(cmd)
+    found_locales, message = _get_installed_language_pack_locales()
     found_packages_locales, message = get_installed_packages_locale(locale)
     locale_data = {}
     messages = message.split("\n")
@@ -365,7 +384,7 @@ def get_language_pack(locale: str) -> tuple:
                         pkg_name = name.replace(".json", "")
                         json_path = os.path.join(root, name)
                         try:
-                            with open(json_path, "r", encoding='utf-8') as fh:
+                            with open(json_path, encoding="utf-8") as fh:
                                 merged_data = json.load(fh)
                         except Exception:
                             messages.append(traceback.format_exc())
@@ -416,6 +435,7 @@ class TranslationBundle:
                 mod = importlib.import_module(language_pack_module)
                 localedir = os.path.join(os.path.dirname(mod.__file__), LOCALE_DIR)
             except Exception:
+                # no-op
                 pass
 
         gettext.bindtextdomain(self._domain, localedir=localedir)
@@ -456,7 +476,7 @@ class TranslationBundle:
         """
         return gettext.dngettext(self._domain, msgid, msgid_plural, n)
 
-    def pgettext(self, msgctxt: str, singular: str) -> str:
+    def pgettext(self, msgctxt: str, msgid: str) -> str:
         """
         Translate a singular string with context.
 
@@ -472,7 +492,14 @@ class TranslationBundle:
         str
             The translated string.
         """
-        return gettext.dpgettext(self._domain, msgctxt, msgid)
+        # Python 3.7 or lower does not offer translations based on context.
+        # On these versions `pgettext` falls back to `gettext`
+        if PY37_OR_LOWER:
+            translation = gettext.dgettext(self._domain, msgid)
+        else:
+            translation = gettext.dpgettext(self._domain, msgctxt, msgid)
+
+        return translation
 
     def npgettext(self, msgctxt: str, msgid: str, msgid_plural: str, n: int) -> str:
         """
@@ -494,7 +521,14 @@ class TranslationBundle:
         str
             The translated string.
         """
-        return gettext.dnpgettext(self._domain, msgctxt, msgid, msgid_plural, n)
+        # Python 3.7 or lower does not offer translations based on context.
+        # On these versions `npgettext` falls back to `ngettext`
+        if PY37_OR_LOWER:
+            translation = gettext.dngettext(self._domain, msgid, msgid_plural, n)
+        else:
+            translation = gettext.dnpgettext(self._domain, msgctxt, msgid, msgid_plural, n)
+
+        return translation
 
     # Shorthands
     def __(self, msgid: str) -> str:
@@ -531,7 +565,7 @@ class TranslationBundle:
         str
             The translated string.
         """
-        return self.ngettext(msgid, plural, n)
+        return self.ngettext(msgid, msgid_plural, n)
 
     def _p(self, msgctxt: str, msgid: str) -> str:
         """
@@ -551,7 +585,7 @@ class TranslationBundle:
         """
         return self.pgettext(msgctxt, msgid)
 
-    def _np(self, msgctxt: str, msgid: str, msgid_plular: str, n: str) -> str:
+    def _np(self, msgctxt: str, msgid: str, msgid_plural: str, n: int) -> str:
         """
         Shorthand for npgettext.
 
@@ -571,13 +605,14 @@ class TranslationBundle:
         str
             The translated string.
         """
-        return self.npgettext(msgctxt, msgid, msgid_plular, n)
+        return self.npgettext(msgctxt, msgid, msgid_plural, n)
 
 
 class translator:
     """
     Translations manager.
     """
+
     _TRANSLATORS = {}
     _LOCALE = DEFAULT_LOCALE
 
@@ -594,6 +629,22 @@ class translator:
         for key in ["LANGUAGE", "LANG"]:
             os.environ[key] = f"{locale}.UTF-8"
 
+    @staticmethod
+    def normalize_domain(domain: str) -> str:
+        """Normalize a domain name.
+
+        Parameters
+        ----------
+        domain: str
+            Domain to normalize
+
+        Returns
+        -------
+        str
+            Normalized domain
+        """
+        return domain.replace("-", "_")
+
     @classmethod
     def set_locale(cls, locale: str):
         """
@@ -604,10 +655,14 @@ class translator:
         locale: str
             The language name to use.
         """
+        if locale == cls._LOCALE:
+            # Nothing to do bail early
+            return
+
         if is_valid_locale(locale):
             cls._LOCALE = locale
             translator._update_env(locale)
-            for domain, bundle in cls._TRANSLATORS.items():
+            for _, bundle in cls._TRANSLATORS.items():
                 bundle.update_locale(locale)
 
     @classmethod
@@ -621,20 +676,87 @@ class translator:
         ----------
         domain: str
             The translations domain. The normalized python package name.
-        
+
         Returns
         -------
         Translator
             A translator instance bound to the domain.
         """
-        if domain in cls._TRANSLATORS:
-            trans = cls._TRANSLATORS[domain]
+        norm_domain = translator.normalize_domain(domain)
+        if norm_domain in cls._TRANSLATORS:
+            trans = cls._TRANSLATORS[norm_domain]
         else:
-            trans = TranslationBundle(domain, cls._LOCALE)
-            cls._TRANSLATORS[domain] = trans
+            trans = TranslationBundle(norm_domain, cls._LOCALE)
+            cls._TRANSLATORS[norm_domain] = trans
 
         return trans
 
+    @staticmethod
+    def _translate_schema_strings(
+        translations,
+        schema: dict,
+        prefix: str = "",
+        to_translate: Dict[Pattern, str] = None,
+    ) -> None:
+        """Translate a schema in-place."""
+        if to_translate is None:
+            to_translate = _prepare_schema_patterns(schema)
+
+        for key, value in schema.items():
+            path = prefix + "/" + key
+
+            if isinstance(value, str):
+                matched = False
+                for pattern, context in to_translate.items():  # noqa
+                    if pattern.fullmatch(path):
+                        matched = True
+                        break
+                if matched:
+                    schema[key] = translations.pgettext(context, value)
+            elif isinstance(value, dict):
+                translator._translate_schema_strings(
+                    translations,
+                    value,
+                    prefix=path,
+                    to_translate=to_translate,
+                )
+            elif isinstance(value, list):
+                for i, element in enumerate(value):
+                    if not isinstance(element, dict):
+                        continue
+                    translator._translate_schema_strings(
+                        translations,
+                        element,
+                        prefix=path + "[" + str(i) + "]",
+                        to_translate=to_translate,
+                    )
+
+    @staticmethod
+    def translate_schema(schema: Dict) -> Dict:
+        """Translate a schema.
+
+        Parameters
+        ----------
+        schema: dict
+            The schema to be translated
+
+        Returns
+        -------
+        Dict
+            The translated schema
+        """
+        if translator._LOCALE == DEFAULT_LOCALE:
+            return schema
+
+        translations = translator.load(
+            schema.get(_lab_i18n_config, {}).get("domain", DEFAULT_DOMAIN)
+        )
+
+        new_schema = schema.copy()
+        translator._translate_schema_strings(translations, schema.copy())
+
+        return new_schema
+
 
 if __name__ == "__main__":
     _main()
